@@ -10,7 +10,6 @@ import (
 	"encoding/json"
 	"fmt"
 	"io"
-	"log"
 	"os"
 	"path"
 	"regexp"
@@ -27,12 +26,14 @@ import (
 	"github.com/rclone/rclone/fs/config/flags"
 	"github.com/rclone/rclone/fs/hash"
 	"github.com/rclone/rclone/fs/object"
+	"github.com/rclone/rclone/fs/operations"
 	"github.com/rclone/rclone/lib/random"
 	"github.com/spf13/cobra"
 )
 
 var (
 	writeJSON          string
+	keepTestFiles      bool
 	checkNormalization bool
 	checkControl       bool
 	checkLength        bool
@@ -56,15 +57,16 @@ func init() {
 	flags.BoolVarP(cmdFlags, &checkStreaming, "check-streaming", "", false, "Check uploads with indeterminate file size", "")
 	flags.BoolVarP(cmdFlags, &checkBase32768, "check-base32768", "", false, "Check can store all possible base32768 characters", "")
 	flags.BoolVarP(cmdFlags, &all, "all", "", false, "Run all tests", "")
+	flags.BoolVarP(cmdFlags, &keepTestFiles, "keep-test-files", "", false, "Keep test files after execution", "")
 }
 
 var commandDefinition = &cobra.Command{
 	Use:   "info [remote:path]+",
 	Short: `Discovers file name or other limitations for paths.`,
-	Long: `rclone info discovers what filenames and upload methods are possible
-to write to the paths passed in and how long they can be.  It can take some
-time.  It will write test files into the remote:path passed in.  It outputs
-a bit of go code for each one.
+	Long: `Discovers what filenames and upload methods are possible to write to the
+paths passed in and how long they can be.  It can take some time.  It will
+write test files into the remote:path passed in.  It outputs a bit of go
+code for each one.
 
 **NB** this can create undeletable files and other hazards - use with care
 `,
@@ -74,7 +76,7 @@ a bit of go code for each one.
 	Run: func(command *cobra.Command, args []string) {
 		cmd.CheckArgs(1, 1e6, command, args)
 		if !checkNormalization && !checkControl && !checkLength && !checkStreaming && !checkBase32768 && !all {
-			log.Fatalf("no tests selected - select a test or use --all")
+			fs.Fatalf(nil, "no tests selected - select a test or use --all")
 		}
 		if all {
 			checkNormalization = true
@@ -84,7 +86,15 @@ a bit of go code for each one.
 			checkBase32768 = true
 		}
 		for i := range args {
-			f := cmd.NewFsDir(args[i : i+1])
+			tempDirName := "rclone-test-info-" + random.String(8)
+			tempDirPath := path.Join(args[i], tempDirName)
+			f := cmd.NewFsDir([]string{tempDirPath})
+			fs.Infof(f, "Created temporary directory for test files: %s", tempDirPath)
+			err := f.Mkdir(context.Background(), "")
+			if err != nil {
+				fs.Fatalf(nil, "couldn't create temporary directory: %v", err)
+			}
+
 			cmd.Run(false, false, command, func() error {
 				return readInfo(context.Background(), f)
 			})
@@ -279,7 +289,7 @@ func (r *results) checkControls() {
 
 	// Concurrency control
 	tokens := make(chan struct{}, ci.Checkers)
-	for i := 0; i < ci.Checkers; i++ {
+	for range ci.Checkers {
 		tokens <- struct{}{}
 	}
 	var wg sync.WaitGroup
@@ -471,10 +481,18 @@ func (r *results) checkStreaming() {
 }
 
 func readInfo(ctx context.Context, f fs.Fs) error {
-	err := f.Mkdir(ctx, "")
-	if err != nil {
-		return fmt.Errorf("couldn't mkdir: %w", err)
+	// Ensure cleanup unless --keep-test-files is specified
+	if !keepTestFiles {
+		defer func() {
+			err := operations.Purge(ctx, f, "")
+			if err != nil {
+				fs.Errorf(f, "Failed to purge temporary directory: %v", err)
+			} else {
+				fs.Infof(f, "Removed temporary directory for test files: %s", f.Root())
+			}
+		}()
 	}
+
 	r := newResults(ctx, f)
 	if checkControl {
 		r.checkControls()
